package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/expression"
)

// ExpressionToRegister puts the result of an expression into the specified register.
func (f *Function) ExpressionToRegister(node *expression.Expression, register cpu.Register) error {
	if node.IsLeaf() {
		return f.TokenToRegister(node.Token, register)
	}

	if ast.IsFunctionCall(node) {
		err := f.CompileCall(node)

		if register != f.cpu.Output[0] {
			f.assembler.RegisterRegister(asm.MOVE, register, f.cpu.Output[0])
		}

		return err
	}

	left := node.Children[0]
	right := node.Children[1]
	final := register

	if OverwritesRegister(right, register) {
		register = f.cpu.MustFindFree(f.cpu.General)
	}

	err := f.ExpressionToRegister(left, register)

	if err != nil {
		return err
	}

	f.cpu.Use(register)
	err = f.Execute(node.Token, register, right)

	if register != final {
		f.assembler.RegisterRegister(asm.MOVE, final, register)
	}

	f.cpu.Free(register)
	return err
}
