package cli

import (
	"fmt"
	"os"
	"strings"

	"git.akyoto.dev/cli/q/src/build"
	"git.akyoto.dev/cli/q/src/build/config"
)

// Build parses the arguments and creates a build.
func Build(args []string) int {
	b := build.New()

	for i := 0; i < len(args); i++ {
		switch args[i] {
		case "-a", "--assembler":
			config.Assembler = true
		case "-c", "--comments":
			config.Comments = true
		case "-d", "--dry":
			config.Dry = true
		case "-v", "--verbose":
			config.Assembler = true
			config.Comments = true
		default:
			if strings.HasPrefix(args[i], "-") {
				fmt.Printf("Unknown parameter: %s\n", args[i])
				return 2
			}

			b.Files = append(b.Files, args[i])
		}
	}

	if len(b.Files) == 0 {
		b.Files = append(b.Files, ".")
	}

	return run(b)
}

// run starts the build by running the compiler and then writing the result to disk.
func run(b *build.Build) int {
	result, err := b.Run()

	if err != nil {
		fmt.Fprintln(os.Stderr, err)
		return 1
	}

	if config.Assembler {
		result.PrintInstructions()
	}

	if config.Dry {
		return 0
	}

	err = result.Write(b.Executable())

	if err != nil {
		fmt.Fprintln(os.Stderr, err)
		return 1
	}

	return 0
}
